// animal1.cpp
// This program shows some deficits of compile-time polymorphism
// with the function void favoriteFood(void).
// We have a superclass animal and subclasses cow and pig. All classes have their own
// implementation of void favoriteFood(void).
// The main function contains an array of animals. 
// M. Schukat, 2002

// Header files
#include <string.h>
#include <stdlib.h>
#include <iostream.h>


// Class definition for animal
class animal
{
protected:
	// Pointer to a name
	char* name;

public:
	// Constructor
	animal(char* s)
	{
		name = new char[strlen(s) + 1];
		strcpy (name, s);
	}

	// Destructor
	~animal(void)
	{
		delete [] name;
	}

	// Function void favoriteFood(void) versus virtual void favoriteFood(void)
	void favoriteFood(void) {cout << "Favorite food of " << name <<": None." << endl;}
};


// Class definition for pig
class pig : public animal
{
public:
	// Constructor: we call the constructor of the superclass.
	pig(char* s) : animal(s) {}

	// Function favoriteFood()
	void favoriteFood(void) {cout << "Favorite food of " << name <<": Apples." << endl;}
};


// Class definition for cow
class cow : public animal
{
public:
	// Constructor: we call the constructor of the superclass.
	cow(char* s) : animal(s) {}

	void favoriteFood(void) {cout << "Favorite food of " << name << ": Hay." << endl;}
};


// Main function

int main(void)
{
	// In order to deal with a set of pigs and cows, we define an array of pointers of the
	// base class animal. The individual pointers will point to cow and pig objects.
	animal* myAnimal[3];

	// We can do this assignment, because cow and pig are subclasses of animal. It is a form
	// of casting
	myAnimal[0] = new cow("Cow1");
	myAnimal[1] = new cow("Cow2");
	myAnimal[2] = new pig("Pig1");

	myAnimal[0]->favoriteFood();
	myAnimal[1]->favoriteFood();
	myAnimal[2]->favoriteFood();

	return 0;
}